<?php

/**
* Copyright Maarch since 2008 under license.
* See LICENSE.txt file at the root folder for more details.
* This file is part of Maarch software.
*
*/

/**
* @brief Docserver Model
* @author dev@maarch.org
*/
namespace Docserver\models;

use SrcCore\models\DatabaseModel;
use SrcCore\models\ValidatorModel;

class DocserverModel
{
    public static function get(array $args)
    {
        ValidatorModel::notEmpty($args, ['select']);
        ValidatorModel::arrayType($args, ['select', 'where', 'data']);

        $docServers = DatabaseModel::select([
            'select'    => $args['select'],
            'table'     => ['docservers'],
            'where'     => empty($args['where']) ? [] : $args['where'],
            'data'      => empty($args['data']) ? [] : $args['data']
        ]);

        if (empty($docServers)) {
            return [];
        }
        return $docServers;
    }

    public static function getByType(array $args)
    {
        ValidatorModel::notEmpty($args, ['type']);
        ValidatorModel::stringType($args, ['type']);
        ValidatorModel::arrayType($args, ['select']);

        $aDocserver = DatabaseModel::select([
            'select'    => empty($args['select']) ? ['*'] : $args['select'],
            'table'     => ['docservers'],
            'where'     => ['type = ?'],
            'data'      => [$args['type']]
        ]);

        if (empty($aDocserver[0])) {
            return [];
        }

        return $aDocserver[0];
    }

    public static function getByTypes(array $args)
    {
        ValidatorModel::notEmpty($args, ['types']);
        ValidatorModel::arrayType($args, ['types']);
        ValidatorModel::arrayType($args, ['select']);

        $docServers = DatabaseModel::select([
            'select'    => empty($args['select']) ? ['*'] : $args['select'],
            'table'     => ['docservers'],
            'where'     => ['type IN (?)'],
            'data'      => [$args['types']]
        ]);

        if (empty($docServers)) {
            return [];
        }
        return $docServers;
    }

    public static function update(array $args)
    {
        ValidatorModel::notEmpty($args, ['set', 'where', 'data']);
        ValidatorModel::arrayType($args, ['set', 'where', 'data']);

        DatabaseModel::update([
            'table' => 'docservers',
            'set'   => $args['set'],
            'where' => $args['where'],
            'data'  => $args['data']
        ]);

        return true;
    }
}
