<?php

/**
* Copyright Maarch since 2008 under license.
* See LICENSE.txt file at the root folder for more details.
* This file is part of Maarch software.
*
*/

/**
* @brief Document Model
* @author dev@maarch.org
*/

namespace Document\models;

use SrcCore\models\ValidatorModel;
use SrcCore\models\DatabaseModel;

class DocumentModel
{
    public static function get(array $args)
    {
        ValidatorModel::notEmpty($args, ['select']);
        ValidatorModel::arrayType($args, ['select', 'where', 'data', 'orderBy']);
        ValidatorModel::intType($args, ['limit', 'offset']);

        $args['where'] = empty($args['where']) ? [] : $args['where'];
        $args['data']  = empty($args['data']) ? [] : $args['data'];

        $aDocuments = DatabaseModel::select([
            'select'    => $args['select'],
            'table'     => ['main_documents'],
            'where'     => $args['where'],
            'data'      => $args['data'],
            'orderBy'   => empty($args['orderBy']) ? [] : $args['orderBy'],
            'offset'    => empty($args['offset']) ? 0 : $args['offset'],
            'limit'     => empty($args['limit']) ? 0 : $args['limit'],
        ]);

        return $aDocuments;
    }

    public static function getById(array $args)
    {
        ValidatorModel::notEmpty($args, ['select', 'id']);
        ValidatorModel::arrayType($args, ['select']);

        $where = ['id = ?'];
        $data  = [$args['id']];

        $document = DatabaseModel::select([
            'select' => $args['select'],
            'table'  => ['main_documents'],
            'where'  => $where,
            'data'   => $data
        ]);

        if (empty($document[0])) {
            return [];
        }

        return $document[0];
    }

    public static function create(array $args)
    {
        ValidatorModel::notEmpty($args, ['title', 'sender', 'metadata', 'typist']);
        ValidatorModel::stringType($args, ['title', 'reference', 'description', 'sender', 'deadline', 'notes', 'link_id', 'metadata', 'mailing_id']);

        $nextSequenceId = DatabaseModel::getNextSequenceValue(['sequenceId' => 'main_documents_id_seq']);

        DatabaseModel::insert([
            'table'         => 'main_documents',
            'columnsValues' => [
                'id'            => $nextSequenceId,
                'title'         => $args['title'],
                'reference'     => $args['reference'],
                'description'   => $args['description'],
                'sender'        => $args['sender'],
                'deadline'      => $args['deadline'],
                'notes'         => $args['notes'],
                'link_id'       => $args['link_id'],
                'metadata'      => $args['metadata'],
                'mailing_id'    => $args['mailing_id'],
                'typist'        => $args['typist']
            ]
        ]);

        return $nextSequenceId;
    }

    public static function update(array $args)
    {
        ValidatorModel::notEmpty($args, ['set', 'where', 'data']);
        ValidatorModel::arrayType($args, ['set', 'where', 'data']);

        DatabaseModel::update([
            'table' => 'main_documents',
            'set'   => $args['set'],
            'where' => $args['where'],
            'data'  => $args['data']
        ]);

        return true;
    }

    public static function delete(array $args)
    {
        ValidatorModel::notEmpty($args, ['where', 'data']);
        ValidatorModel::arrayType($args, ['where', 'data']);

        DatabaseModel::delete([
            'table' => 'main_documents',
            'where' => $args['where'],
            'data'  => $args['data']
        ]);

        return true;
    }
}
