<?php

/**
* Copyright Maarch since 2008 under license.
* See LICENSE.txt file at the root folder for more details.
* This file is part of Maarch software.
*
*/

/**
* @brief Notification Stack Model
* @author dev@maarch.org
*/

namespace Notification\models;

use SrcCore\models\ValidatorModel;
use SrcCore\models\DatabaseModel;

class NotificationStackModel
{
    public static function get(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['select']);
        ValidatorModel::arrayType($aArgs, ['select', 'where', 'data', 'orderBy']);
        ValidatorModel::intType($aArgs, ['limit', 'offset']);

        $stack = DatabaseModel::select([
            'select'    => $aArgs['select'],
            'table'     => ['notifications_stack'],
            'where'     => empty($aArgs['where']) ? [] : $aArgs['where'],
            'data'      => empty($aArgs['data']) ? [] : $aArgs['data'],
            'orderBy'   => empty($aArgs['orderBy']) ? [] : $aArgs['orderBy'],
            'offset'    => empty($aArgs['offset']) ? 0 : $aArgs['offset'],
            'limit'     => empty($aArgs['limit']) ? 0 : $aArgs['limit'],
        ]);

        return $stack;
    }

    public static function getById(array $args)
    {
        ValidatorModel::notEmpty($args, ['select', 'id']);
        ValidatorModel::arrayType($args, ['select']);

        $stack = DatabaseModel::select([
            'select' => $args['select'],
            'table'  => ['notifications_stack'],
            'where'  => ['id = ?'],
            'data'   => [$args['id']]
        ]);

        if (empty($stack[0])) {
            return [];
        }

        return $stack[0];
    }

    public static function create(array $args)
    {
        ValidatorModel::notEmpty($args, ['mainDocumentId', 'type', 'recipientId']);
        ValidatorModel::stringType($args, ['type']);
        ValidatorModel::intVal($args, ['mainDocumentId', 'recipientId']);

        $nextSequenceId = DatabaseModel::getNextSequenceValue(['sequenceId' => 'notifications_stack_id_seq']);

        DatabaseModel::insert([
            'table'         => 'notifications_stack',
            'columnsValues' => [
                'id'               => $nextSequenceId,
                'main_document_id' => $args['mainDocumentId'],
                'type'             => $args['type'],
                'recipient_id'     => $args['recipientId']
            ]
        ]);

        return $nextSequenceId;
    }

    public static function update(array $args)
    {
        ValidatorModel::notEmpty($args, ['set', 'where', 'data']);
        ValidatorModel::arrayType($args, ['set', 'where', 'data']);

        DatabaseModel::update([
            'table' => 'notifications_stack',
            'set'   => $args['set'],
            'where' => $args['where'],
            'data'  => $args['data']
        ]);

        return true;
    }

    public static function delete(array $args)
    {
        ValidatorModel::notEmpty($args, ['where', 'data']);
        ValidatorModel::arrayType($args, ['where', 'data']);

        DatabaseModel::delete([
            'table' => 'notifications_stack',
            'where' => $args['where'],
            'data'  => $args['data']
        ]);

        return true;
    }
}
