<?php

/**
* Copyright Maarch since 2008 under license.
* See LICENSE.txt file at the root folder for more details.
* This file is part of Maarch software.
*
*/

/**
* @brief Notifications Schedule Model
* @author dev@maarch.org
*/

namespace Notification\models;

use SrcCore\models\DatabaseModel;
use SrcCore\models\ValidatorModel;

class NotificationsScheduleModel
{
    public static function get(array $args = [])
    {
        ValidatorModel::arrayType($args, ['select', 'where', 'data', 'orderBy']);
        ValidatorModel::intType($args, ['limit']);

        $notificationsSchedule = DatabaseModel::select([
            'select'    => empty($args['select']) ? ['*'] : $args['select'],
            'table'     => ['notifications_schedule'],
            'where'     => empty($args['where']) ? [] : $args['where'],
            'data'      => empty($args['data']) ? [] : $args['data'],
            'orderBy'   => empty($args['orderBy']) ? [] : $args['orderBy'],
            'limit'     => empty($args['limit']) ? 0 : $args['limit']
        ]);

        return $notificationsSchedule;
    }

    public static function getById(array $args)
    {
        ValidatorModel::notEmpty($args, ['id']);
        ValidatorModel::intVal($args, ['id']);
        ValidatorModel::arrayType($args, ['select']);

        $notificationsScheduleItem = NotificationsScheduleModel::get([
            'select'    => empty($args['select']) ? ['*'] : $args['select'],
            'where'     => ['id = ?'],
            'data'      => [$args['id']]
        ]);

        if (!empty($notificationsScheduleItem)) {
            return $notificationsScheduleItem[0];
        }

        return [];
    }

    public static function create(array $args)
    {
        ValidatorModel::notEmpty($args, ['label', 'type']);
        ValidatorModel::stringType($args, ['label', 'type', 'frequency_mode', 'start_time', 'end_time', 'months', 'days_of_month', 'days_of_week']);
        ValidatorModel::intType($args, ['frequency']);

        $nextSequenceId = DatabaseModel::getNextSequenceValue(['sequenceId' => 'notifications_schedule_id_seq']);
        DatabaseModel::insert([
            'table'         => 'notifications_schedule',
            'columnsValues' => [
                'id'             => $nextSequenceId,
                'label'          => $args['label'],
                'type'           => $args['type'],
                'months'         => $args['months'],
                'days_of_month'  => $args['days_of_month'],
                'days_of_week'   => $args['days_of_week'],
                'start_time'     => $args['start_time'],
                'end_time'       => $args['end_time'],
                'frequency'      => $args['frequency'],
                'frequency_mode' => $args['frequency_mode'],
            ],
        ]);

        return $nextSequenceId;
    }

    public static function update(array $args)
    {
        ValidatorModel::notEmpty($args, ['set', 'where', 'data']);
        ValidatorModel::arrayType($args, ['set', 'where', 'data']);

        DatabaseModel::update([
            'table' => 'notifications_schedule',
            'set'   => $args['set'],
            'where' => $args['where'],
            'data'  => $args['data']
        ]);

        return true;
    }

    public static function delete(array $args)
    {
        ValidatorModel::notEmpty($args, ['where', 'data']);
        ValidatorModel::arrayType($args, ['where', 'data']);

        DatabaseModel::delete([
            'table' => 'notifications_schedule',
            'where' => $args['where'],
            'data'  => $args['data']
        ]);

        return true;
    }
}
