import { Component, OnInit, ViewChild } from '@angular/core';
import { SignaturesContentService } from '../../../service/signatures.service';
import { NotificationService } from '../../../service/notification.service';
import { HttpClient } from '@angular/common/http';
import { MatDialog } from '@angular/material/dialog';
import { MatSidenav } from '@angular/material/sidenav';
import { map, finalize, tap, catchError } from 'rxjs/operators';
import { ActivatedRoute, Router } from '@angular/router';
import { ConfirmComponent } from '../../../plugins/confirm.component';
import { TranslateService } from '@ngx-translate/core';
import { AuthService } from '../../../service/auth.service';
import { ModalController } from '@ionic/angular';
import { Portal } from '@angular/cdk/portal';
import { of } from 'rxjs';
import { FunctionsService } from '../../../service/functions.service';


export interface Cas {
    id: number;
    label: string;
    identifier: string;
    value: {
        version: string;
        url: string;
        port: number;
        context: string;
        separator: number;
        certificate: string;
    };
}

@Component({
    selector: 'app-administration-cas',
    templateUrl: 'cas.component.html',
    styleUrls: ['../../administration.scss'],
})

export class CasComponent implements OnInit {

    creationMode: boolean = true;
    loading: boolean = true;
    loadingTest: boolean = false;
    casTest: any = {
        login: '',
        password: '',
        result: ''
    };
    cas: Cas = {
        id: null,
        label: '',
        identifier: 'casServer',
        value: {
            version: '',
            url: '',
            port: null,
            context: '',
            separator: null,
            certificate: '',
        }
    };
    casClone: Cas;
    title: string = '';

    constructor(
        public http: HttpClient,
        private translate: TranslateService,
        private route: ActivatedRoute,
        private router: Router,
        public signaturesService: SignaturesContentService,
        public notificationService: NotificationService,
        public dialog: MatDialog,
        public authService: AuthService,
        public modalController: ModalController,
        public functions: FunctionsService,
    ) {
    }

    ngOnInit(): void {
        this.casTest.login = this.authService.user.login;
        this.creationMode = false;
        this.http.get('../rest/configurations?identifier=casServer')
            .pipe(
                map((data: any) => data.configurations[0]),
                tap ((data: any) => {
                    if (!this.functions.empty(data)) {
                        this.cas = {
                            id: data.id,
                            label: data.label,
                            identifier: data.identifier,
                            value: {
                                version: data?.value?.version,
                                url: data?.value?.url,
                                port: data?.value?.port,
                                context: data?.value?.context,
                                separator: data?.value?.separator,
                                certificate: data?.value?.certificate,
                            }
                        };
                        this.casClone = JSON.parse(JSON.stringify(this.cas));
                        this.title = this.cas.label;
                    }
                }),

                finalize(() => this.loading = false)
            ).subscribe();
    }

    canValidate() {
        return JSON.stringify(this.cas) !== JSON.stringify(this.casClone);
    }

    onSubmit() {
        this.modifyCas();
    }

    modifyCas() {
        if (this.cas.id !== null) {
            this.http.patch('../rest/configurations/' + this.cas.id, this.cas)
                .pipe(
                    finalize(() => {
                        this.loading = false;
                        this.router.navigate(['/administration/connections']);
                        this.notificationService.success('lang.casUpdated');
                    }),
                    catchError((err: any) => {
                        this.notificationService.handleErrors(err);
                        return of(false);
                    })
                )
                .subscribe();
        } else {
            this.createCas();
        }
    }

    createCas() {
        this.loading = true;
        this.http.post('../rest/configurations', this.cas)
            .pipe(
                finalize(() => {
                    this.loading = false;
                    this.router.navigate(['/administration/connections']);
                    this.notificationService.success('lang.casAdded');
                }),
                catchError((err: any) => {
                    this.notificationService.handleErrors(err);
                    return of(false);
                })
            ).subscribe();
    }

    cancel() {
        this.router.navigate(['/administration/connections']);
    }
    isNumber(evt: any) {
        evt = (evt) ? evt : window.event;
        const charCode = (evt.which) ? evt.which : evt.keyCode;
        if (charCode > 31 && (charCode < 48 || charCode > 57)) {
            return false;
        }
        return true;
    }

}
