import { Component, OnInit, ViewChild } from '@angular/core';
import { NotificationService } from '../../service/notification.service';
import { HttpClient } from '@angular/common/http';
import { TranslateService } from '@ngx-translate/core';
import { AuthService } from '../../service/auth.service';
import { FunctionsService } from '../../service/functions.service';
import { ActivatedRoute, Router } from '@angular/router';
import { of } from 'rxjs';
import { catchError, finalize, map, tap } from 'rxjs/operators';
import { DatePipe } from '@angular/common';
import { IonAccordionGroup } from '@ionic/angular';

@Component({
    templateUrl: 'notification.component.html',
    styleUrls: ['../administration.scss', 'notification.component.scss'],
    providers: [DatePipe]
})

export class NotificationComponent implements OnInit {

    @ViewChild('startTimeAccordionGroup', { static: false }) startTimeAccordionGroup: IonAccordionGroup;
    @ViewChild('endTimeAccordionGroup', { static: false }) endTimeAccordionGroup: IonAccordionGroup;

    loading: boolean = true;
    creationMode: boolean = true;

    notification: any = {
        id: null,
        label: '',
        type: 'next_user',
        startTime: null,
        daysOfWeek: [],
        months: [],
        daysOfMonth: [],
        frequency: null,
        frequencyMode: 'hours',
        endTime: null,
    };

    frequencyModes: [
        'minutes',
        'hours',
    ];

    days = [
        {
            id: 1,
            label : 'lang.monday'
        },
        {
            id: 2,
            label : 'lang.tuesday'
        },
        {
            id: 3,
            label : 'lang.wednesday'
        },
        {
            id: 4,
            label : 'lang.thursday'
        },
        {
            id: 5,
            label : 'lang.friday'
        },
        {
            id: 6,
            label : 'lang.saturday'
        },
        {
            id: 7,
            label : 'lang.sunday'
        }
    ];

    months = [
        {
            id: 1,
            label : 'lang.january'
        },
        {
            id: 2,
            label : 'lang.february'
        },
        {
            id: 3,
            label : 'lang.march'
        },
        {
            id: 4,
            label : 'lang.april'
        },
        {
            id: 5,
            label : 'lang.may'
        },
        {
            id: 6,
            label : 'lang.june'
        },
        {
            id: 7,
            label : 'lang.july'
        },
        {
            id: 8,
            label : 'lang.august'
        },
        {
            id: 9,
            label : 'lang.september'
        },
        {
            id: 10,
            label : 'lang.october'
        },
        {
            id: 11,
            label : 'lang.november'
        },
        {
            id: 12,
            label : 'lang.december'
        }
    ];

    notificationTypes: any[] = [
        {
            id: 'next_user',
            label: this.translate.instant('lang.notification_next_user')
        },
        {
            id: 'typist_END',
            label: this.translate.instant('lang.notification_typist_END')
        },
        {
            id: 'typist_INT',
            label: this.translate.instant('lang.notification_typist_INT')
        },
        {
            id: 'typist_REF',
            label: this.translate.instant('lang.notification_typist_REF')
        }
    ];

    daysMonth: any = [];

    test: any = [];

    currentFrequencyMode = 'daily';
    repeatFrequency: boolean = false;

    constructor(
        private route: ActivatedRoute,
        public http: HttpClient,
        private router: Router,
        private translate: TranslateService,
        public notificationService: NotificationService,
        public authService: AuthService,
        public functions: FunctionsService,
        private datePipe: DatePipe
    ) { }

    ngOnInit() {
        this.genDaysOfMonth();
        this.route.params.subscribe(async (params: any) => {
            if (params['id'] === undefined) {
                this.loading = false;
            } else {
                this.creationMode = false;
                await this.getSchedule(params['id']);
                this.loading = false;
            }
        });
    }

    onSubmit() {
        if (this.currentFrequencyMode === 'daily') {
            this.notification.months = ['*'];
            this.notification.daysOfMonth = ['*'];
            this.notification.daysOfWeek = this.notification.daysOfWeek.filter((item: any) => item !== '*');
        } else if (this.currentFrequencyMode === 'monthly'){
            this.notification.months = this.notification.months.filter((item: any) => item !== '*');
            this.notification.daysOfMonth = this.notification.daysOfMonth.filter((item: any) => item !== '*');
            this.notification.daysOfWeek = ['*'];
        }
        this.notification.frequency =  this.repeatFrequency ? this.notification.frequency : null;
        this.notification.endTime =  this.repeatFrequency ? this.notification.endTime : null;

        if (this.creationMode) {
            this.createSchedule();
        } else {
            this.updateSchedule();
        }
    }

    formatdata() {
        const formatData = JSON.parse(JSON.stringify(this.notification));

        const objStartTime = new Date(formatData.startTime);

        if (objStartTime.toString() !== 'Invalid Date') {
            formatData.startTime = this.datePipe.transform(formatData.startTime, 'HH:mm');
        }

        if (!this.functions.empty(formatData.endTime)) {
            const objEndTime = new Date(formatData.endTime);
            if (objEndTime.toString() !== 'Invalid Date') {
                formatData.endTime = this.datePipe.transform(formatData.endTime, 'HH:mm');
            }
        }
        formatData.lastExecution = null;
        return formatData;
    }

    getSchedule(id: number) {
        return new Promise((resolve) => {
            this.http.get(`../rest/schedule/${id}`).pipe(
                map((data: any) => data.notificationsScheduleItem),
                tap((data: any) => {
                    this.notification = data;
                    if (this.notification.months[0] === '*' && this.notification.daysOfMonth[0] === '*') {
                        this.currentFrequencyMode = 'daily';
                    } else {
                        this.currentFrequencyMode = 'monthly';
                    }
                    if (!this.functions.empty(this.notification.endTime)) {
                        this.repeatFrequency = true;
                    }
                }),
                finalize(() => resolve(true)),
                catchError((err: any) => {
                    this.notificationService.handleErrors(err);
                    return of(false);
                })
            ).subscribe();
        });
    }

    createSchedule() {
        return new Promise((resolve) => {
            this.http.post('../rest/schedule', this.formatdata()).pipe(
                tap(() => {
                    this.notificationService.success('lang.scheduleAdded');
                    this.router.navigate(['/administration/notifications']);
                }),
                finalize(() => resolve(true)),
                catchError((err: any) => {
                    this.notificationService.handleErrors(err);
                    return of(false);
                })
            ).subscribe();
        });
    }

    updateSchedule() {
        return new Promise((resolve) => {
            this.http.put(`../rest/schedule/${this.notification.id}`, this.formatdata()).pipe(
                tap(() => {
                    this.notificationService.success('lang.scheduleUpdated');
                    this.router.navigate(['/administration/notifications']);
                }),
                finalize(() => resolve(true)),
                catchError((err: any) => {
                    this.notificationService.handleErrors(err);
                    return of(false);
                })
            ).subscribe();
        });
    }

    genDaysOfMonth() {
        for (let index = 1; index <= 31; index++) {
            this.daysMonth.push({
                id: index,
                label: index
            });
        }
    }

    frequencyChanged(mode: string) {
        this.currentFrequencyMode = mode;
    }

    toggleRepeatFrequency(ev: any) {
        if (ev.detail.checked) {
            if (this.functions.empty(this.notification.frequency)) {
                this.notification.frequency = 1;
            }

            if (this.functions.empty(this.notification.endTime)) {
                this.notification.endTime = '23:59';
            }
        }
    }

    checkEndTime(startTime: any) {
        if (!this.functions.empty(this.notification.endTime)) {
            const startDate = new Date(startTime);
            const endDate = new Date(this.notification.endTime);
            if (startDate < endDate) {
                this.notification.endTime = startTime;
            }
        }

        if (this.getDate(this.notification.startTime) === '23:59') {
            this.repeatFrequency = false;
        }
    }

    getDate(date: string): string {
        return !this.creationMode ? date : this.datePipe.transform(date, 'HH:mm');
    }

    getValues(target: string): number[] {
        const startTime: any[] = this.getDate(this.notification.startTime).split(':');
        const values: number[] = [];
        if (this.getDate(this.notification.startTime) === '22:59') {
            this.notification.endTime = '23:00';
            return target === 'hours' ? [23] : [0o0];
        }
        if (target === 'hours') {
            for (let i = startTime[0]; i <= 23; i ++) {
                values.push(i);
            }
            return values;
        } else if (target === 'minutes') {
            for (let i = startTime[1]; i <= 59; i ++) {
                values.push(i);
            }
            return values;
        }
    }

    toggleAccordion(target: string, event: Event = null) {
        if (event?.type === 'ionChange' && this.notification[target] === undefined) {
            this.notification[target] = new Date().toISOString().toString();
        }
        this[`${target}AccordionGroup`].value = undefined;
    }
}
