import { NgModule } from '@angular/core';
import { RouterModule } from '@angular/router';
import { AuthGuard } from './service/auth.guard';

import { AdministrationComponent } from './administration/home/administration.component';
import { UsersListComponent } from './administration/user/users-list.component';
import { UserComponent } from './administration/user/user.component';
import { GroupsListComponent } from './administration/group/groups-list.component';
import { GroupComponent } from './administration/group/group.component';
import { ConnectionComponent } from './administration/connection/connection.component';
import { LdapListComponent } from './administration/connection/ldap/ldap-list.component';
import { LdapComponent } from './administration/connection/ldap/ldap.component';
import { SendmailComponent } from './administration/sendmail/sendmail.component';
import { DocumentComponent } from './document/document.component';
import { LoginComponent } from './login/login.component';
import { ForgotPasswordComponent } from './login/forgotPassword/forgotPassword.component';
import { UpdatePasswordComponent } from './login/updatePassword/updatePassword.component';
import { SecuritiesAdministrationComponent } from './administration/security/securities-administration.component';
import { PasswordModificationComponent } from './login/passwordModification/password-modification.component';
import { ProfileComponent } from './profile/profile.component';
import { HomeComponent } from './home/home.component';
import { IndexationComponent } from './indexation/indexation.component';
import { SearchComponent } from './search/search.component';
import { HistoryListComponent } from './administration/history/history-list.component';
import { OtpListComponent } from './administration/otp/otp-list.component';
import { OtpComponent } from './administration/otp/otp.component';
import { CustomizationComponent } from './administration/customization/customization.component';
import { NotificationComponent } from './administration/notification/notification.component';
import { NotificationsListComponent } from './administration/notification/notifications-list.component';
import { CasComponent } from './administration/connection/cas/cas.component';

@NgModule({
    imports: [
        RouterModule.forRoot([
            { path: 'home', canActivate: [AuthGuard], component: HomeComponent },
            { path: 'indexation', canActivate: [AuthGuard], component: IndexationComponent },
            { path: 'profile', canActivate: [AuthGuard], component: ProfileComponent },
            { path: 'search', canActivate: [AuthGuard], component: SearchComponent },
            { path: 'administration', canActivate: [AuthGuard], component: AdministrationComponent },
            { path: 'administration/users', canActivate: [AuthGuard], component: UsersListComponent },
            { path: 'administration/users/new', canActivate: [AuthGuard], component: UserComponent },
            { path: 'administration/users/:id', canActivate: [AuthGuard], component: UserComponent },
            { path: 'administration/groups', canActivate: [AuthGuard], component: GroupsListComponent },
            { path: 'administration/groups/new', canActivate: [AuthGuard], component: GroupComponent },
            { path: 'administration/groups/:id', canActivate: [AuthGuard], component: GroupComponent },
            { path: 'administration/connections', canActivate: [AuthGuard], component: ConnectionComponent },
            { path: 'administration/connections/ldaps', canActivate: [AuthGuard], component: LdapListComponent },
            { path: 'administration/connections/cas', canActivate: [AuthGuard], component: CasComponent },
            { path: 'administration/connections/ldaps/new', canActivate: [AuthGuard], component: LdapComponent },
            { path: 'administration/connections/ldaps/:id', canActivate: [AuthGuard], component: LdapComponent },
            { path: 'administration/emailConfiguration', canActivate: [AuthGuard], component: SendmailComponent },
            { path: 'administration/passwordRules', canActivate: [AuthGuard], component: SecuritiesAdministrationComponent },
            { path: 'administration/history', canActivate: [AuthGuard], component: HistoryListComponent },
            { path: 'administration/otps', canActivate: [AuthGuard], component: OtpListComponent },
            { path: 'administration/otps/new', canActivate: [AuthGuard], component: OtpComponent },
            { path: 'administration/otps/:id', canActivate: [AuthGuard], component: OtpComponent },
            { path: 'administration/customization', canActivate: [AuthGuard], component: CustomizationComponent},
            { path: 'administration/notifications', canActivate: [AuthGuard], component: NotificationsListComponent},
            { path: 'administration/notifications/new', canActivate: [AuthGuard], component: NotificationComponent},
            { path: 'administration/notifications/:id', canActivate: [AuthGuard], component: NotificationComponent},
            { path: 'documents/:id', canActivate: [AuthGuard], component: DocumentComponent },
            { path: 'login', canActivate: [AuthGuard], component: LoginComponent },
            { path: 'forgot-password', component: ForgotPasswordComponent },
            { path: 'update-password', component: UpdatePasswordComponent },
            { path: 'password-modification', component: PasswordModificationComponent },
            { path: 'externalSignatoryBookReturn/token/:token?return=:status', component: LoginComponent},
            { path: '**', redirectTo: 'login', pathMatch: 'full' },
        ], { relativeLinkResolution: 'legacy', useHash: true }),
    ],
    exports: [
        RouterModule
    ]
})
export class AppRoutingModule { }
