import { Component, OnInit, ViewChild, TemplateRef, ViewContainerRef, Injectable, OnDestroy } from '@angular/core';
import { SignaturesContentService } from '../service/signatures.service';
import { DomSanitizer } from '@angular/platform-browser';
import { MatBottomSheet, MatBottomSheetConfig } from '@angular/material/bottom-sheet';
import { MatDialog } from '@angular/material/dialog';
import { MatSidenav } from '@angular/material/sidenav';
import { SignaturesComponent } from '../signatures/signatures.component';
import { ActivatedRoute, Router } from '@angular/router';
import { HttpClient } from '@angular/common/http';
import { NotificationService } from '../service/notification.service';
import { CookieService } from 'ngx-cookie-service';
import { DocumentNotePadComponent } from '../documentNotePad/document-note-pad.component';
import { RejectInfoBottomSheetComponent } from '../modal/reject-info.component';
import { TranslateService } from '@ngx-translate/core';
import { DocumentListComponent } from './document-list/document-list.component';
import { AuthService } from '../service/auth.service';
import { LocalStorageService } from '../service/local-storage.service';
import { ActionSheetController, AlertController, LoadingController, MenuController, ModalController, NavController } from '@ionic/angular';
import { NgxExtendedPdfViewerService } from 'ngx-extended-pdf-viewer';
import { catchError, exhaustMap, tap } from 'rxjs/operators';
import { of, Subscription, timer } from 'rxjs';
import { SignatureMethodService } from '../service/signature-method/signature-method.service';
import { FunctionsService } from '../service/functions.service';
import { ActionsService } from '../service/actions.service';
import { SuccessInfoValidBottomSheetComponent } from '../modal/success-info-valid.component';

@Component({
    selector: 'app-document',
    templateUrl: 'document.component.html',
    styleUrls: ['document.component.scss'],
})


export class DocumentComponent implements OnInit, OnDestroy {

    @ViewChild('mainContent') mainContent: any;
    @ViewChild('img') img: any;
    @ViewChild('snav', { static: true }) snav: MatSidenav;
    @ViewChild('dragElem') dragElem: any;
    @ViewChild('appDocumentNotePad') appDocumentNotePad: DocumentNotePadComponent;
    @ViewChild('appDocumentList') appDocumentList: DocumentListComponent;
    @ViewChild('rightContent', { static: true }) rightContent: TemplateRef<any>;
    @ViewChild('pagesList') pagesList: any;

    actionsList: any = [
        {
            id: 2,
            label: 'lang.reject',
            color: 'danger',
            logo: 'thumbs-down-outline',
            event: 'refuseDocument'
        },
        {
            id: 3,
            label: 'lang.signatures',
            color: '',
            logo: '',
            event: 'openSignatures'
        },
        {
            id: 1,
            label: 'lang.validate',
            color: 'success',
            logo: 'thumbs-up-outline',
            event: 'validateDocument'
        },
    ];

    mainDocument: any = {
        id: 0,
        status,
        attachments: [],
        workflow: [],
    };

    subscription: Subscription;
    timerSubscription: Subscription;

    signaturesContent: any = [];
    docList: any = [];

    posX: number = 0;
    posY: number = 0;
    pageNum: number = 1;
    currentDoc: number = 0;
    totalPages: number;

    detailMode: boolean = false;
    draggable: boolean;
    expandedNote: boolean = true;
    dragging: boolean = false;
    resizing: boolean = false;
    loadingdocument: boolean = true;
    loadingpdf: boolean = false;
    loadingImage: boolean = true;
    hasWorkflowNotes: boolean = false;

    currentTool: string = 'info';
    pdfname: string = null;
    fileContent: string = '';
    userMode: string = '';

    load: HTMLIonLoadingElement = null;

    constructor(
        public http: HttpClient,
        public signaturesService: SignaturesContentService,
        public notificationService: NotificationService,
        public sanitizer: DomSanitizer,
        public dialog: MatDialog,
        public authService: AuthService,
        public actionSheetController: ActionSheetController,
        public loadingController: LoadingController,
        public viewContainerRef: ViewContainerRef,
        public modalController: ModalController,
        public alertController: AlertController,
        public signatureMethodService: SignatureMethodService,
        public navCtrl: NavController,
        public functionsService: FunctionsService,
        public actionsService: ActionsService,
        private translate: TranslateService,
        private router: Router,
        private route: ActivatedRoute,
        private bottomSheet: MatBottomSheet,
        private localStorage: LocalStorageService,
        private menu: MenuController,
        private pdfViewerService: NgxExtendedPdfViewerService,
        private cookieService: CookieService,
    ) {
        this.draggable = false;
        this.subscription = this.actionsService.catchEvent().subscribe((event: any) => {
            if (event.id === 'gotoDocument' && event.documentIndex === 0) {
                this.loadDoc(event.documentIndex);
                this.actionsService.setEvent('scrollToTop');
            }
        });
    }

    imageLoaded(ev: any) {
        this.userMode = this.mainDocument.workflow.find((item: any) => item.current)?.mode;
        if (this.userMode === 'note') {
            this.actionsList = this.actionsList.filter((item: any) => item.event !== 'openSignatures');
        }
        this.getImageDimensions(true);
        if (this.mainDocument.status !== 'CONVERTING') {
            this.load.dismiss();
        }
        this.menu.enable(true, 'right-menu');
        this.loadingImage = false;
        document.getElementsByClassName('drag-scroll-content')[0].scrollTop = 0;
    }

    getImageDimensions(ajustSize: boolean = false): void {
        const img = new Image();
        img.onload = (data: any) => {
            const percent = (data.target.naturalWidth * 100) / this.signaturesService.workingAreaWidth;

            this.signaturesService.workingAreaWidth = data.target.naturalWidth;
            this.signaturesService.workingAreaHeight = data.target.naturalHeight;

            if (ajustSize) {
                this.getAreaDimension();
            }
        };
        img.src = this.docList[this.currentDoc].imgContent[this.pageNum];
    }

    getAreaDimension() {
        const percent = (this.mainContent.el.offsetWidth * 100) / this.signaturesService.workingAreaWidth;
        this.signaturesService.workingAreaWidth = (percent * this.signaturesService.workingAreaWidth) / 100;
        this.signaturesService.workingAreaHeight = (percent * this.signaturesService.workingAreaHeight) / 100;
    }

    async openAction(event: any) {
        this.posX = event.clientX;
        this.posY = event.clientY;
        const buttons = [];
        if (!this.checkEmptyNote()) {
            buttons.push({
                text: this.translate.instant('lang.cancelPreviousNote'),
                icon: 'arrow-undo-outline',
                handler: () => {
                    this.undoTag();
                }
            });
        }

        if (!this.signaturesService.stampLock) {
            if (this.userMode !== 'note') {
                buttons.push({
                    text: this.translate.instant('lang.affixSignature'),
                    icon: 'ribbon-outline',
                    handler: () => {
                        this.openSignatures();
                    }
                });
            }
            buttons.push({
                text: this.translate.instant('lang.annotateDocument'),
                icon: 'receipt-outline',
                handler: () => {
                    this.openNoteEditor();
                }
            });
        }

        if (!this.checkEmptiness(true, true, true)) {
            buttons.push({
                text: this.translate.instant('lang.deleteAll'),
                icon: 'color-wand-outline',
                handler: () => {
                    this.removeTags();
                }
            });
        }

        const actionSheet = await this.actionSheetController.create({
            header: 'Actions',
            cssClass: 'my-custom-class',
            buttons: buttons,
        });
        await actionSheet.present();
    }

    async openSignatures() {
        const modal = await this.modalController.create({
            component: SignaturesComponent,
            cssClass: 'my-custom-class',
            componentProps: {
                currentWorflow: this.mainDocument.workflow.filter((line: { current: boolean }) => line.current === true)[0],
                content: this.fileContent
            }
        });
        await modal.present();
        const { data } = await modal.onWillDismiss();
        this.dragging = false;
        if (data !== undefined) {
            if (data === 'success') {
                // this.scrollToElem();
            } else if (data.redirectPage !== undefined) {
                this.goTo(data.redirectPage);
            }
        }
    }

    async openNoteEditor() {
        if (this.mainDocument.isCertified && this.userMode === 'note') {
            this.signaturesService.stampLock = true;
            const modal = await this.modalController.create({
                component: SignaturesComponent,
                cssClass: 'my-custom-class',
                componentProps: {
                    currentWorflow: this.mainDocument.workflow.filter((line: { current: boolean }) => line.current === true)[0],
                    content: this.fileContent
                }
            });
            await modal.present();
            await modal.onWillDismiss().then((res: any) => {
                if (res.data === 'cancel') {
                    setTimeout(() => {
                        this.signaturesService.stampLock = false;
                    }, 150);
                }
            });
        } else {
            const modal = await this.modalController.create({
                component: DocumentNotePadComponent,
                cssClass: 'fullscreen',
                componentProps: {
                    precentScrollLeft: this.posX,
                    precentScrollTop: this.posY,
                    content: this.docList[this.currentDoc].imgContent[this.pageNum]
                }
            });
            await modal.present();
            const { data } = await modal.onWillDismiss();
        }
    }

    scrollToElem() {
        const pageY = this.signaturesService.signaturesContent[this.pageNum][this.signaturesService.signaturesContent[this.pageNum].length - 1].positionY;
        const offsetTop = -($('#myBounds')[0].getBoundingClientRect().top - 70);
        const realPosY = (pageY - 75) + offsetTop;

        const scrollY = (realPosY - $(window).height());

        document.getElementsByClassName('drag-scroll-content')[0].scrollTo(1000, -scrollY);
    }

    ionViewWillEnter() {
        this.signaturesService.initTemplate(this.rightContent, this.viewContainerRef, 'rightContent');
    }

    ngOnInit(): void {
        this.menu.enable(false, 'right-menu');
        this.menu.enable(true, 'left-menu');

        this.route.params.subscribe(params => {
            if (typeof params['id'] !== 'undefined') {
                this.loadingController.create({
                    message: this.translate.instant('lang.loadingDocument'),
                    spinner: 'dots'
                }).then((load: HTMLIonLoadingElement) => {
                    this.load = load;
                    this.load.present();
                    this.initDocumentInfos(params);
                });
            }
        });
    }

    ngOnDestroy(): void {
        this.timerSubscription?.unsubscribe();
        this.subscription.unsubscribe();
    }

    initDocumentInfos(params: any) {
        this.http.get('../rest/documents/' + params['id']).pipe(
            tap((data: any) => {
                this.mainDocument = data.document;
                const isSign = /_sign_/g;
                this.mainDocument.workflow = this.mainDocument.workflow.map((item: any) => {
                    if (item.note) {
                        this.hasWorkflowNotes = true;
                    }
                    if (item.userId === null) {
                        if (item.mode.match(isSign) !== null) {
                            item.signatureMode = item.mode;
                        } else if (item.mode === 'sign' && item.signatureMode === 'stamp') {
                            item.signatureMode = 'sign';
                        }
                    }
                    let modes = ['visa', 'sign'].concat(item.userSignatureModes);
                    if (item.signatureMode === 'otp_sign_yousign') {
                        modes = modes.concat(['otp_sign_yousign']);
                    }
                    if (item.signatureMode === 'otp_visa_yousign') {
                        modes = modes.concat(['otp_visa_yousign']);
                    }
                    if (item.signatureMode === 'otp_sign_fast') {
                        modes = modes.concat(['otp_sign_fast']);
                    }
                    return {
                        ...item,
                        role: item.mode === 'visa' ? 'visa' : item.signatureMode,
                        modes: modes
                    };
                });

                this.totalPages = this.mainDocument.pages;
                this.signaturesService.mainDocumentId = this.mainDocument.id;
                this.signaturesService.totalPage = this.mainDocument.pages;
                this.menu.enable(true, 'right-menu');
                this.initDoc();

                const realUserWorkflow = this.mainDocument.workflow.filter((line: { current: boolean }) => line.current === true);

                this.mainDocument.isCertified = this.mainDocument.workflow.filter((line: any) => line.status !== 'REF' && line.status !== 'STOP' && line.mode.indexOf('sign') > -1 && line.signatureMode !== 'stamp' && line.processDate !== null).length > 0;
                const externalUser: any = this.mainDocument.workflow.filter((user: any) => user.userId === null && user.current === true);
                if (realUserWorkflow.length === 0 || this.mainDocument.readOnly || externalUser.length > 0) {
                    this.actionsList = [
                        {
                            id: 4,
                            label: 'lang.back',
                            color: 'medium',
                            logo: 'chevron-back-outline',
                            event: 'back'
                        },
                    ];
                    this.detailMode = true;
                } else {
                    this.signaturesService.stampLock = this.mainDocument.isCertified && ((realUserWorkflow[0].signatureMode === 'stamp' && realUserWorkflow[0].mode === 'sign') || (realUserWorkflow[0].mode === 'visa'));
                    if (realUserWorkflow[0].userId !== this.authService.user.id) {
                        this.http.get('../rest/users/' + realUserWorkflow[0].userId + '/signatures')
                            .subscribe((dataSign: any) => {
                                this.signaturesService.signaturesListSubstituted = dataSign.signatures;
                            });
                    } else {
                        this.signaturesService.signaturesListSubstituted = [];
                    }
                    if (realUserWorkflow[0].datePositions.length > 0 && this.functionsService.empty(this.signaturesService.datesContent)) {
                        realUserWorkflow[0].datePositions.forEach((date: any) => {
                            if (!this.signaturesService.datesContent[date.page]) {
                                this.signaturesService.datesContent[date.page] = [];
                            }
                            this.signaturesService.datesContent[date.page][0] = {
                                width: date.width,
                                height: date.height,
                                positionX: date.positionX,
                                positionY: date.positionY,
                                font: date.font,
                                size: date.size,
                                color: date.color,
                                format: date.format
                            };
                        });
                    }
                }

                this.docList.push({ 'id': this.mainDocument.id, 'title': this.mainDocument.title, 'pages': this.mainDocument.pages, 'imgContent': [], 'imgUrl': '../rest/documents/' + this.mainDocument.id + '/thumbnails' });
                this.mainDocument.attachments.forEach((attach: any) => {
                    this.docList.push({ 'id': attach.id, 'title': attach.title, 'pages': attach.pages, 'imgContent': [], 'imgUrl': '../rest/attachments/' + attach.id + '/thumbnails' });
                });
                this.menu.enable(true, 'right-menu');
                // this.renderPdf();
                this.renderImage();
                this.loadingdocument = false;
                this.load.dismiss();

                if (this.mainDocument.status === 'CONVERTING') {
                    // timer(0, 10000) call the function immediately and every 10 seconds
                    this.timerSubscription = timer(0, 10000).pipe(
                        tap(() => {
                            this.http.get('../rest/documents/' + params['id']).pipe(
                                tap((res: any) => {
                                    this.totalPages = res.document.pages;
                                    if (res.document.status !== 'CONVERTING') {
                                        this.mainDocument.status = res.document.status;
                                        this.timerSubscription?.unsubscribe();
                                    }
                                })
                            ).subscribe();
                        }),
                        catchError((err: any) => {
                            this.load.dismiss();
                            this.notificationService.handleErrors(err);
                            return of(false);
                        })
                    ).subscribe();
                }
            }),
            catchError((err: any) => {
                console.log('error', err);
                setTimeout(() => {
                    this.load.dismiss();
                }, 200);
                this.notificationService.handleErrors(err);
                this.router.navigate(['/home']);
                return of(false);
            })
        ).subscribe();
    }

    renderPdf() {
        this.http.get('../rest/documents/' + this.docList[this.currentDoc].id + '/content').pipe(
            tap((data: any) => {
                this.pdfname = 'data:application/pdf;base64,' + data.encodedDocument;
                this.loadingpdf = true;
            }),
            catchError((err: any) => {
                this.notificationService.handleErrors(err);
                return of(false);
            })
        ).subscribe();
    }

    async onPagesLoaded(ev: any) {
        this.totalPages = ev.pagesCount;
        this.exportAsImage();
    }

    public async exportAsImage(): Promise<void> {
        const scale = { width: 1000 };
        // or: scale = {height: this.height};
        // or: scale = {scale: this.scale};
        const data = await this.pdfViewerService.getPageAsImage(this.pageNum, scale);
        this.docList[this.currentDoc].imgContent[this.pageNum] = data;
        this.loadingpdf = false;
        this.load.dismiss();
    }

    renderImage() {
        if (this.docList[this.currentDoc].imgContent[this.pageNum] === undefined) {
            if (this.currentDoc === 0) {
                this.http.get('../rest/documents/' + this.docList[this.currentDoc].id + '/thumbnails/' + this.pageNum).pipe(
                    tap((data: any) => {
                        this.docList[this.currentDoc].imgContent[this.pageNum] = 'data:image/png;base64,' + data.fileContent;
                        this.fileContent = data.fileContent;
                    }),
                    catchError((err: any) => {
                        this.load.dismiss();
                        this.notificationService.handleErrors(err);
                        this.router.navigate(['/home']);
                        return of(false);
                    })
                ).subscribe();
            } else {
                this.http.get('../rest/attachments/' + this.docList[this.currentDoc].id + '/thumbnails/' + this.pageNum).pipe(
                    tap((data: any) => {
                        this.docList[this.currentDoc].imgContent[this.pageNum] = 'data:image/png;base64,' + data.fileContent;
                    }),
                    catchError((err: any) => {
                        this.load.dismiss();
                        this.notificationService.handleErrors(err);
                        this.router.navigate(['/home']);
                        return of(false);
                    })
                ).subscribe();
            }
        }
    }

    initDoc() {
        this.docList = [];
        this.signaturesService.signaturesContent = [];
        this.signaturesService.notesContent = [];
        this.signaturesService.datesContent = [];
        this.signaturesService.currentToobal = 'mainDocumentDetail';

        const notesContent = this.localStorage.get(this.mainDocument.id.toString());

        if (notesContent) {
            const storageContent = JSON.parse(notesContent);
            this.signaturesService.notesContent = storageContent['note'] !== undefined ? storageContent['note'] : [];
            this.signaturesService.signaturesContent = storageContent['sign'] !== undefined ? storageContent['sign'] : [];
            this.signaturesService.datesContent = storageContent['date'] !== undefined ? storageContent['date'] : [];
        }

        this.signaturesService.currentAction = 0;
        this.signaturesService.currentPage = 1;
        this.pageNum = 1;
        this.signaturesContent.currentDoc = 1;
        this.currentDoc = 0;
    }

    prevPage() {
        this.loadingImage = true;
        this.pageNum--;

        if (this.pageNum === 0) {
            this.pageNum = 1;
        }

        if (this.currentDoc === 0) {
            this.signaturesService.currentPage = this.pageNum;
        }
        // this.exportAsImage();
        this.renderImage();
    }

    nextPage() {
        this.loadingImage = true;
        if (this.pageNum >= this.totalPages) {
            this.pageNum = this.totalPages;
        } else {
            this.pageNum++;
        }

        // only for main document
        if (this.currentDoc === 0) {
            this.signaturesService.currentPage = this.pageNum;
        }
        // this.exportAsImage();
        this.renderImage();
    }

    goTo(page: number) {
        this.loadingController.create({
            message: this.translate.instant('lang.loadingDocument'),
            spinner: 'dots',
        }).then((load: HTMLIonLoadingElement) => {
            this.load = load;
            this.load.present();
            this.loadingImage = true;
            this.load.dismiss();
        });
        this.pageNum = page;
        // only for main document
        if (this.currentDoc === 0) {
            this.signaturesService.currentPage = this.pageNum;
        }
        // this.exportAsImage();
        this.renderImage();
    }

    pagesArray(n: number): number[] {
        return Array(n);
    }

    async refuseDocument(): Promise<void> {
        let msg: string = this.translate.instant('lang.rejectDocumentWarning');
        const mode: string = this.mainDocument.workflow.find((item: any) => item.current).mode;

        if (this.signaturesService.notesContent.length === 0) {
            msg = this.translate.instant('lang.refuseDocumentWithoutNote');
        }

        const alert = await this.alertController.create({
            cssClass: 'custom-alert-danger',
            header: this.translate.instant('lang.reject'),
            message: msg,
            inputs: [
                {
                    name: 'paragraph',
                    id: 'paragraph',
                    type: 'textarea',
                    placeholder: this.translate.instant('lang.addReason')
                },
            ],
            buttons: [
                {
                    text: this.translate.instant('lang.reject'),
                    handler: async (data: any) => {
                        this.signaturesService.signaturesContent = [];
                        const idsToProcess = await this.actionsService.checkGroupMail(this.mainDocument, 'reject');
                        const res = await this.signatureMethodService.launchDefaultMode(data.paragraph, idsToProcess);

                        if (!this.functionsService.empty(res)) {
                            const config: MatBottomSheetConfig = {
                                disableClose: true,
                                direction: 'ltr'
                            };
                            this.bottomSheet.open(RejectInfoBottomSheetComponent, config);
                            this.localStorage.remove(this.mainDocument.id.toString());
                        }
                    }
                }
            ]
        });
        await alert.present();
    }

    async validateDocument(mode: any): Promise<void> {
        let msg = this.userMode !== 'note' ? this.translate.instant('lang.validateDocumentWarning') : this.translate.instant('lang.validateDocumentWarningNote');
        const event: any = this.actionsList.find((item: any) => item.event === 'validateDocument');
        if (this.signaturesService.signaturesContent.length === 0 && this.signaturesService.notesContent.length === 0) {
            msg = this.translate.instant('lang.validateDocumentWithoutSignOrNote');
        }
        if (this.signaturesService.stampLock) {
            msg = this.translate.instant('lang.certifiedDocumentMsg2');
        }
        const alert = await this.alertController.create({
            cssClass: 'custom-alert-success',
            header: this.getLabel(event),
            message: msg,
            inputs: [
                {
                    name: 'paragraph',
                    id: 'paragraph',
                    type: 'textarea',
                    placeholder: this.translate.instant('lang.addReason')
                },
            ],
            buttons: [
                {
                    text: this.getLabel(event),
                    handler: async (data: any) => {
                        await this.processToSign(data);
                    }
                }
            ]
        });

        await alert.present();
    }

    async processToSign(data: any) {
        const currentUserWorkflow = this.mainDocument.workflow.filter((line: { current: boolean }) => line.current === true)[0];
        const idsToProcess = await this.actionsService.checkGroupMail(this.mainDocument, 'validate');
        const res = await this.signatureMethodService.checkAuthenticationAndLaunchAction(currentUserWorkflow, data.paragraph, idsToProcess);
        if (!this.functionsService.empty(res)) {
            if (res === 'refresh_certs') {
                this.processToSign(data);
            } else {
                const config: MatBottomSheetConfig = {
                    disableClose: true,
                    direction: 'ltr'
                };
                this.bottomSheet.open(SuccessInfoValidBottomSheetComponent, config);
                this.localStorage.remove(this.mainDocument.id.toString());
            }
        }
    }

    async removeTags() {
        this.signaturesService.currentAction = 0;

        const alert = await this.alertController.create({
            header: this.translate.instant('lang.deleteNoteAndSignature'),
            buttons: [
                {
                    text: this.translate.instant('lang.validate'),
                    handler: () => {
                        this.signaturesService.signaturesContent = [];
                        this.signaturesService.notesContent = [];
                        this.signaturesService.datesContent = [];
                        this.localStorage.remove(this.mainDocument.id.toString());
                        this.notificationService.success('lang.noteAndSignatureDeleted');
                    }
                }
            ]
        });

        await alert.present();
    }

    loadDoc(index: any) {
        this.pageNum = 1;
        this.currentDoc = index;
        this.totalPages = this.docList[index].pages;
    }

    launchEvent(action: any) {
        this.backToDetails();
        this.signaturesService.currentAction = action.id;
        this[action.event]();
    }

    undoTag() {
        if (this.signaturesService.notesContent[this.pageNum]) {
            this.signaturesService.notesContent[this.pageNum].pop();
            this.localStorage.remove(this.mainDocument.id.toString());
            this.localStorage.save(this.mainDocument.id.toString(), JSON.stringify({ 'sign': this.signaturesService.signaturesContent, 'note': this.signaturesService.notesContent }));
            if (this.checkEmptiness(false, true, false)) {
                this.signaturesService.notesContent = [];
            }
        }
    }

    checkEmptyNote() {
        if (!this.signaturesService.notesContent[this.pageNum]) {
            return true;
        } else if (this.signaturesService.notesContent[this.pageNum] === 'undefined') {
            return true;
        } else if (this.signaturesService.notesContent[this.pageNum].length === 0) {
            return true;
        } else {
            return false;
        }
    }

    checkEmptiness(checkDate: boolean, checkSignature: boolean, checkNote: boolean) {
        let state = true;
        for (let pageNum = 1; pageNum <= this.signaturesService.totalPage; pageNum++) {
            if (this.signaturesService.datesContent[pageNum] && checkDate) {
                if (this.signaturesService.datesContent[pageNum].length > 0) {
                    state = false;
                    break;
                }
            }
            if (this.signaturesService.notesContent[pageNum] && checkNote) {
                if (this.signaturesService.notesContent[pageNum].length > 0) {
                    state = false;
                    break;
                }
            }
            if (this.signaturesService.signaturesContent[pageNum] && checkSignature) {
                if (this.signaturesService.signaturesContent[pageNum].length > 0) {
                    state = false;
                    break;
                }
            }
        }
        return state;
    }

    openVisaWorkflow() {
        this.menu.open('right-menu');
        this.signaturesService.currentToobal = 'visaWorkflow';
    }

    openDocumentList() {
        this.menu.open('right-menu');
        this.signaturesService.currentToobal = 'documentList';
    }

    openAssociatedDocuments() {
        this.menu.open('right-menu');
        this.signaturesService.currentToobal = 'associatedDocuments';
    }

    openMainDocumentDetail() {
        this.menu.open('right-menu');
        this.signaturesService.currentToobal = 'mainDocumentDetail';
    }

    backToDetails() {
        this.signaturesService.currentToobal = 'mainDocumentDetail';
    }

    back() {
        this.router.navigate(['/search']);
    }

    ionViewWillLeave() {
        this.signaturesService.detachTemplate('rightContent');
        // To resolve overlay error
        this.load.dismiss();
    }

    openSelect(event: any) {
        if (this.totalPages > 1) {
            this.pagesList.interface = 'popover';
            this.pagesList.open(event);
        }
    }

    canShowActions() {
        return this.currentDoc === 0 && !this.detailMode && !this.signaturesService.stampLock;
    }

    getLabel(action: any) {
        if (action.event !== 'validateDocument') {
            return this.translate.instant(action.label);
        } else {
            const mode: string = this.mainDocument.workflow.find((item: any) => item.current).mode;
            switch (mode) {
                case 'sign':
                    return this.translate.instant('lang.validateSign');
                case 'visa':
                    return this.translate.instant('lang.validate');
                case 'note':
                    return this.translate.instant('lang.transmit');
            }
        }
    }

    isNotReady() {
        return ['CONVERTING', 'ERROR'].indexOf(this.mainDocument.status) > -1;
    }
}
