import { Component, EventEmitter, Input, OnInit, Output, ViewChild } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { NotificationService } from '../../../../service/notification.service';
import { TranslateService } from '@ngx-translate/core';
import { NgForm } from '@angular/forms';
import { OtpService } from '../../../../service/otp-connectors/otp-generator.service';
import { FunctionsService } from '../../../../service/functions.service';
import { OtpFieldsFast } from '../../../../service/otp-connectors/otp-fast.service';

@Component({
    selector: 'app-otp-fast',
    templateUrl: 'otp-fast.component.html',
    styleUrls: ['otp-fast.component.scss'],
})
export class OtpFastComponent implements OnInit {

    @ViewChild('otpForm', { static: false }) otpForm: NgForm;

    @Input() connectorId: number;
    @Input() otpFast: any;

    otpFields: OtpFieldsFast = null;

    constructor(
        public http: HttpClient,
        public notificationService: NotificationService,
        public otpService: OtpService,
        private functions: FunctionsService,
        private translate: TranslateService
    ) {

    }

    async ngOnInit(): Promise<void> {
        this.otpFields = this.otpService.getOtpFields();
        this.otpFields.sourceId = this.otpFast ? this.otpFast.sourceId : this.connectorId;
        await this.getConfig(this.connectorId);
    }

    async getConfig(id: number = this.connectorId) {
        if (this.otpFast) {
            this.otpFields = {...this.otpFields, ...this.otpFast};
            this.otpFields.sourceId = this.connectorId;
        }
    }

    getData() {
        if (!this.functions.empty(this.otpFields.phone)) {
            this.otpService.formatPhone();
        }
        return this.otpFields;
    }

    isValid() {
        return this.otpForm?.valid;
    }
}
