import {  Component, Input, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { NotificationService } from '../../../service/notification.service';
import { AuthService } from '../../../service/auth.service';
import { TranslateService } from '@ngx-translate/core';
import { ModalController } from '@ionic/angular';
import { OtpYousignComponent } from './yousign/otp-yousign.component';
import { OtpService } from '../../../service/otp-connectors/otp-generator.service';
import { FunctionsService } from '../../../service/functions.service';
import { OtpFastComponent } from './fast/otp-fast.component';

@Component({
    selector: 'app-otp-create',
    templateUrl: 'otp-create.component.html',
    styleUrls: ['otp-create.component.scss'],
})
export class OtpCreateComponent implements OnInit {

    @ViewChild('appOtpComponent', { read: ViewContainerRef, static: false }) appOtpComponent: ViewContainerRef;

    @Input() data: any;

    otpComponent: any;
    currentSource: any = null;

    sourcesByType: any[] = [];
    connectors: any[] = [];
    allSources: any[] = [];

    connectorId: number;

    loading: boolean = true;
    canChooseConnectorType: boolean = true;

    selectedConnector: string = '';

    constructor(
        public http: HttpClient,
        public notificationService: NotificationService,
        public authService: AuthService,
        public modalController: ModalController,
        public otpService: OtpService,
        public functions: FunctionsService,
        public translate: TranslateService
    ) { }

    async ngOnInit() {
        if (!this.functions.empty(this.data?.type)) {
            this.setConnector({id: this.data.type});
        }
        this.getConfig();
    }

    async getConfig() {
        this.allSources = await this.otpService.getConnectorsConfig(this.selectedConnector);
        if (!this.functions.empty(this.allSources)) {
            this.sourcesByType = this.allSources;
            const filterArray: any[] = [... new Set(this.sourcesByType.map((item: any) => item.type))];
            if (filterArray.length > 1) {
                for (let i = 0; i < this.otpService.otpConnectorTypes.length; i ++) {
                    this.otpService.otpConnectorTypes[i].logo = await this.otpService.getUserOtpIcon(this.otpService.otpConnectorTypes[i].id);
                    this.connectors.push(this.otpService.otpConnectorTypes[i]);
                }
                this.connectors = [... new Set(this.connectors)];
            } else if (filterArray.length === 1) {
                this.canChooseConnectorType = false;
                this.selectedConnector = filterArray[0];
                this.otpService.setConnector({id: this.selectedConnector});
                this.connectorId = this.allSources.find((connector: any) => connector.type === filterArray[0]).id;
                setTimeout(() => {
                    this.createComponent();
                }, 100);
            }
        }
        this.loading = false;
    }

    onSubmit() {
        this.modalController.dismiss(this.otpComponent.instance.getData());
    }

    dismissModal() {
        this.modalController.dismiss('cancel');
    }

    isValid(): boolean {
        return this.otpComponent?.instance?.isValid();
    }

    setCurrerentSource(id: any) {
        this.otpComponent.instance.connectorId = id;
        this.connectorId = id;
        this.otpComponent?.instance.getConfig(id);
    }

    setConnector(connector: any) {
        this.selectedConnector = connector.id;
        this.otpService.setConnector(connector);
        this.sourcesByType = this.allSources.filter((item: any) => item.type === this.selectedConnector);
        if (this.functions.empty(this.data?.type)) {
            this.currentSource = this.sourcesByType[0];
            this.connectorId = this.currentSource.id;
        } else {
            this.currentSource = {
                sourceId : this.data.sourceId,
                type: this.data.type
            };
            this.connectorId = this.currentSource.sourceId;
        }
        setTimeout(() => {
            this.createComponent();
        }, 100);
    }

    createComponent() {
        if (this.functions.empty(this.data)) {
            this.otpService.resetFieldsValues();
        }
        this.appOtpComponent?.clear();
        if (this.selectedConnector === 'yousign') {
            this.otpComponent = this.appOtpComponent?.createComponent(OtpYousignComponent);
            if (this.otpComponent) {
                this.otpComponent.instance.connectorId = this.connectorId;
                this.otpComponent.instance.otpYousign = this.data;
            }
        } else if (this.selectedConnector === 'fast')  {
            this.otpComponent = this.appOtpComponent?.createComponent(OtpFastComponent);
            if (this.otpComponent) {
                this.otpComponent.instance.connectorId = this.connectorId;
                this.otpComponent.instance.otpFast = this.data;
            }
        }
    }

    getTitle(): string {
        return this.selectedConnector !==  '' ? `${this.translate.instant('lang.otpUser')} - ${this.translate.instant('lang.' + this.selectedConnector + 'Otp')}`  :  this.translate.instant('lang.otpUser');
    }
}
