import { Component, Input, OnInit, ViewChild } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { NotificationService } from '../../../../service/notification.service';
import { TranslateService } from '@ngx-translate/core';
import { NgForm } from '@angular/forms';
import { OtpService } from '../../../../service/otp-connectors/otp-generator.service';
import { FunctionsService } from '../../../../service/functions.service';
import { OtpFieldsYousign } from '../../../../service/otp-connectors/otp-yousign.service';

@Component({
    selector: 'app-otp-yousign',
    templateUrl: 'otp-yousign.component.html',
    styleUrls: ['otp-yousign.component.scss'],
})
export class OtpYousignComponent implements OnInit {
    @ViewChild('otpForm', { static: false }) otpForm: NgForm;

    @Input() connectorId: number;
    @Input() otpYousign: any;

    otpFields: OtpFieldsYousign = null;

    constructor(
        public http: HttpClient,
        private translate: TranslateService,
        public notificationService: NotificationService,
        public otpService: OtpService,
        private functions: FunctionsService
    ) {
    }

    async ngOnInit(): Promise<void> {
        this.otpFields = this.otpService.getOtpFields();
        this.otpFields.sourceId = this.otpYousign ? this.otpYousign.sourceId : this.connectorId;
        await this.getConfig(this.connectorId);
    }

    async getConfig(id: number = this.connectorId) {
        const result: any = await this.otpService.getConnectorById(id);
        if (!this.functions.empty(result)) {
            this.otpFields.securityModes = [... new Set(result.otp.securityModes)] as string[];
            this.otpFields.security = this.otpFields.securityModes[0];
            if (this.otpYousign) {
                this.otpFields = {...this.otpFields, ...this.otpYousign};
                this.otpFields.sourceId = this.connectorId;
            }
        }
    }

    getData() {
        if (!this.functions.empty(this.otpFields?.phone)) {
            this.otpService.formatPhone();
        }
        return this.otpFields;
    }

    getSecurityMode() {
        return this.translate.instant('lang.' + this.otpFields?.security);
    }

    isValid() {
        return this.otpForm?.valid;
    }
}
