import { HttpClient } from '@angular/common/http';
import { Component, OnInit } from '@angular/core';
import { ModalController } from '@ionic/angular';
import { TranslateService } from '@ngx-translate/core';
import { of } from 'rxjs';
import { catchError, tap } from 'rxjs/operators';
import { environment } from '../../../core/environments/environment';
import { FunctionsService } from '../../service/functions.service';
import { NotificationService } from '../../service/notification.service';



@Component({
    templateUrl: 'about-us.component.html',
    styleUrls: ['about-us.component.scss'],
})

export class AboutUsComponent implements OnInit {

    loading: boolean = false;
    applicationVersion: string;
    currentYear: number;
    commitHash: string = this.translate.instant('lang.undefined');
    license: string = '';
    xeliansLogo: any;
    externalComponents: any[] = [
        {
            label: 'Angular',
            url: 'https://angular.io/'
        },
        {
            label: 'Angular Material',
            url: 'https://material.angular.io/'
        },
        {
            label: 'Ionic',
            url: 'https://ionicframework.com/'
        },
        {
            label: 'Docaposte',
            url: 'https://www.docaposte.com/'
        },
        {
            label: 'YouSign',
            url: 'https://yousign.com/'
        },
        {
            label: 'Font Awesome',
            url: 'https://github.com/FortAwesome/Font-Awesome/'
        },
        {
            label: 'Ionicons',
            url: 'https://ionic.io/ionicons/'
        },
        {
            label: 'Jquery',
            url: 'https://jquery.com/'
        },
        {
            label: 'Jquery Typeahead',
            url: 'https://github.com/running-coder/jquery-typeahead/'
        },
    ];

    constructor (
        public modalController: ModalController,
        public translate: TranslateService,
        public http: HttpClient,
        public functions: FunctionsService,
        private notify: NotificationService,

    ) { }

    async ngOnInit(): Promise<void> {
        this.loading = true;
        this.xeliansLogo = await this.functions.getPictureById('maarch_box');
        this.applicationVersion = environment.VERSION;
        this.license = environment.LICENSE;
        this.currentYear = new Date().getFullYear();
        await this.loadCommitInformation();
        this.loading = false;
    }

    loadCommitInformation() {
        return new Promise((resolve) => {
            this.http.get('../rest/commitInformation').pipe(
                tap((data: any) => {
                    this.commitHash = data.hash !== null ? data.hash : this.translate.instant('lang.undefined');
                    resolve(true);
                }),
                catchError((err: any) => {
                    this.notify.handleErrors(err);
                    return of(false);
                })
            ).subscribe();
        });
    }
}
